/*
 * gline.c
 * (C) Christian Magnusson 2000, Email: d1mag@dtek.chalmers.se
 * This program is copyright, and covered by the Gnu Public License.
 * The Q bot.
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "settings.h"
#include "bottypes.h"
#include "globals.h"
#include "dl.h"

#define MIN(x, y) (((x) < (y)) ? (x) : (y))

void m_gline(char *a);

MODULE_PUBLIC int _init_gline(void *a, void *b)
{
  dl_command_entry_t *dl = a;
  if(!a) return 0;

  snprintf(dl->help, MAX_COMMAND_HELP_LEN,
	   "/msg %s GLINE <user@host> [<duration> s/h/d/w/m/y] <reason>\nAdd a G-line to a host, and refuse connections from that host under some time. Time could be either seconds, hours, days, weeks, months or years.\n", BOT_NAME);
  snprintf(dl->summary, MAX_COMMAND_SUMMARY_LEN,
	   "Add a G-line to a host");
  dl->level = 950;
#if 0
  dl->func = m_gline;
#else
  dl->func = (void (*)(char*)) DL_sym(dl->val, "m_gline");
#endif
  return 1;
}

MODULE_PUBLIC int _remove_gline(char *a, char *b)
{
  remove_command("gline");
  return 1;
}

MODULE_PUBLIC void m_gline(char *a)
{
  struct activeuser *u;
  ListElem *el;
  struct gline *g;
  char tmp[512];
  char time_str[80];
  int start;
  int i, nr;
  char gline[GLINELEN+1];
  char gline_mask[GLINELEN+1];
  char nick[NICKLEN+1];
  char user[USERLEN+1];
  char host[HOSTLEN+1];
  int dur_len;
  int rlen;
  long durationl;
  char duration[21];
  char reason_test[REASONLEN+1];
  char reason[REASONLEN+1];
  struct tm *tm_;
  time_t time_;
  struct dbaseuser *db;
  db = sender_au->dbuser;
  if(db) {
    log("From: %s gline [%s]\n", db->nick, a);
  } else
    return;
  if(!is_oper()) {
    say_sender("You are not an Oper.");
    log("To: %s Is not an Oper\n", db->nick);
    return;
  }

  i=0;
  strncpy(gline, getword(a,&i), GLINELEN);
  gline[GLINELEN] = '\0';

  if(strlen(gline) < 1) {
    say_sender( "You forgot to specify which G-line to add");
    say_sender_( "/msg %s GLINE <user@host> [<duration> s/h/d/w/m/y] <reason>", BOT_NAME);
    return;
  }

  if(!split_banstylename(gline, nick, user, host)) {
    say_sender("Couldn't validate user@host");
    return;
  }

  if(!strcmp(gline, "*@*")) {
    say_sender("G-line shouldn't be *@*");
    return;
  }
  if(!strcmp(user, "*") && !strcmp(host, "*")) {
    say_sender("G-line shouldn't be *@*");
    return;
  }

  if(strlen(gline) < 5) {
    say_sender("G-line too short");
    return;
  }

  /* convert it to correct format ! */
  strncpy(gline_mask, banstylename(nick, user, host), GLINELEN);
  gline_mask[GLINELEN] = '\0';
  
  if((gline_mask[0] != '*') || (gline_mask[1] != '!')) {
    say_sender("G-line should be user@host (not nick!user@host)");
    return;
  }

  strncpy(gline, &gline_mask[2], GLINELEN-2);
  gline[GLINELEN-2] = '\0';

  strncpy(duration, getword(a,&i), 20);
  duration[20] = '\0';
  durationl = atol(duration);
  dur_len = strlen(duration);

  if(dur_len && !isdigit(duration[dur_len-1])) {
    switch(duration[dur_len-1]) {
    case 'h':
      durationl *= 3600;
      break;
    case 'd':
      durationl *= 24*3600;
      break;
    case 'w':
      durationl *= 7*24*3600;
      break;
    case 'm':
      durationl *= 30*24*3600;
      break;
    case 'y':
      durationl *= 365*24*3600;
      break;
    default:
      break;
    }
  }
  if(durationl == 0) {
    say_sender("No duration length was given.");
    return;
  }
  sprintf(duration, "%ld", durationl);

  start = i+1;
  strncpy(reason_test, getword(a,&i), 10);
  reason_test[10] = '\0';
  if(!strlen(reason_test)) {
    say_sender("No reason was given.");
    return;
  }
  strncpy(reason, &a[start], REASONLEN);
  rlen = MIN(strlen(&a[start]), REASONLEN);
  reason[rlen] = '\0';

  g = gline_add(gline, duration, reason, sender_au->userid, 0);
  if(!g) {
    say_sender("Couldn't add the G-line for some reason.");
    return;
  }
  strncpy(g->user, user, USERLEN);
  g->user[USERLEN] = '\0';
  strncpy(g->host, host, HOSTLEN);
  g->host[HOSTLEN] = '\0';

  nr = 0;
  el = GetHead(auser_list);
  while(el) {
    u = (struct activeuser *)el->data;
    if(matches_gline(u, g)) {
      nr++;
    }
    el = GetSucc(el);
  }
  if((nr > 40) && (nr > servers[MAX_SERVERS].users/10)) {
    sprintf(tmp, "You are not allowed to G-line %d users", nr);
    say_sender(tmp);
    gline_del(g->gline);
    return;
  }

  time_ = durationl + time(NULL);
  tm_ = localtime(&time_);
  strftime(time_str, 60, "%c", tm_);
  sprintf(tmp, "GLINE %s expires %s (%d user%s) (set by %s)\n", gline, time_str,
	  nr, ((nr==1) ? "" : "s"), sender_au->dbuser->nick);
  wallop(tmp);
  log(tmp);

  putserver("%c%c GL * +%s %ld :%s\n\r", yxx[0], yxx[1], g->gline, g->duration, g->reason);
  return;
}
